/* Open Source License

Copyright 2005 by Frank McCown
All rights reserved.

Developed by:  Frank McCown
               Old Dominion University
               http://www.cs.odu.edu/~fmccown

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal with the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

         - Redistributions of source code must retain the above copyright
           notice, this list of conditions and the following disclaimers.
         - Redistributions in binary form must reproduce the above copyright
           notice, this list of conditions and the following disclaimers in the
           documentation and/or other materials provided with the distribution.
         - Neither the names of Open Archives Initiative Metadata Harvesting
           Project, Old Dominion University, nor the names of
           its contributors may be used to endorse or promote products derived
           from this Software without specific prior written permission.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS WITH THE SOFTWARE.

*/

using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace oaidemo
{
	/// <summary>
	/// Summary description for ListIdentifiers.
	/// </summary>
	public class ListIdentifiers : System.Web.UI.Page
	{
		protected string errorMessage = null;
		protected string resumptionToken = null;
		protected string metadataPrefix = null;
		protected string untilDate = null;
		protected string fromDate = null;
		protected string setSpec = null;    
		protected string nextResumptionToken = "";
		protected string headers;



		/// <summary>
		/// Make sure only valid parameters were sent.
		/// Optional params: from, until, set
		/// Required: metadataPrefix
		/// Other: resumptionToken (exclusive parameter that cannot be given with any other params)
		/// </summary>
		private void CheckParameters()
		{
			ArrayList paramList = new ArrayList(Request.QueryString.Keys);
			paramList.AddRange(Request.Form.Keys);

			foreach (string parameter in paramList)
			{
				switch (parameter)
				{
					case "verb": break;
					case "from": break;
					case "until": break;
					case "set": break;
					case "metadataPrefix": break;
					case "resumptionToken": break;
					default:
						errorMessage += @"<error code=""badArgument"">Illegal argument '" + parameter + "'.</error>";
						break;
				}
			}


			// Check for resumptionToken
			resumptionToken = Request["resumptionToken"];
			if (resumptionToken != null)
			{
				if (resumptionToken.Length == 0)
				{
					errorMessage += @"<error code=""badArgument"">Content of argument 'resumptionToken' is missing.</error>";
				}
				else if (Request.Form.Count + Request.QueryString.Count > 2)
				{
					errorMessage += @"<error code=""badArgument"">'resumptionToken' must be the only parameter</error>";
				}
			}

			// Check for from, until, set
			fromDate = Request["from"];
			if (fromDate != null && fromDate.Length == 0)
			{
				errorMessage += @"<error code=""badArgument"">Content of argument 'from' is missing.</error>";
			}
			else if (fromDate != null && !Util.ValidDate(fromDate))
			{
				errorMessage += @"<error code=""badArgument"">Date of 'from' must be in YYYY-MM-DD format.</error>";
			}


			untilDate = Request["until"];
			if (untilDate != null && untilDate.Length == 0)
			{
				errorMessage += @"<error code=""badArgument"">Content of argument 'until' is missing.</error>";
			}
			else if (untilDate != null && !Util.ValidDate(untilDate))
			{
				errorMessage += @"<error code=""badArgument"">Date of 'until' must be in YYYY-MM-DD format.</error>";
			}

			setSpec = Request["set"];
			if (setSpec != null) 
			{
				if (setSpec.Length == 0)
				{
					errorMessage += @"<error code=""badArgument"">Content of argument 'set' is missing.</error>";
				}
				else if ((bool)Application["SupportSets"] == false)
				{
					errorMessage += @"<error code=""noSetHierarchy"">Sets are not supported by this repository.</error>";
				}
			}

			
			// Check for metadataPrefix
			metadataPrefix = Request["metadataPrefix"];
			if (metadataPrefix == null && resumptionToken == null)
			{
				// Missing metadataPrefix
				errorMessage += @"<error code=""badArgument"">Required argument 'metadataPrefix' is missing.</error>";
			}
			else if (metadataPrefix != null && metadataPrefix.Length == 0)
			{
				errorMessage += @"<error code=""badArgument"">Content of argument 'metadataPrefix' is missing.</error>";
			}
			else if (metadataPrefix != null && !CheckMetadataPrefix(metadataPrefix))
			{
				errorMessage += @"<error code=""cannotDisseminateFormat"">'" + metadataPrefix + "' is not a valid metadataPrefix.</error>";
			}
		}


		/// <summary>
		/// Retreives identifiers given the optional parameters to the OAI query
		/// </summary>
		/// <param name="prefix">metadataPrefix parameter</param>
		/// <param name="from">from parameter</param>
		/// <param name="until">until parameter</param>
		/// <param name="theSet">set parameter</param>
		/// <param name="resToken">resumptionToken parameter</param>
		private void RetrieveIdentifiers(string prefix, string from, string until, string theSet, string resToken)
		{
			DataSet ds = (DataSet)Application["RecordsDataSet"];
			string dateDelimiter = (string)Application["DateDelimiter"];

			string select = "";

			if (resToken != null)
			{
				string startAtID;

				// Use 12/31/2111 as far-off date for expiration
				// NOTE: This should probably be an off-set from the current time defined by a 
				// parameter in the RepositoryDescription.xml file.

                string error = Util.ParseResumptionTokenWithOaiId("12/31/2111", resToken,
                    out until, out from, out theSet, out prefix, out startAtID);
				if (error.Length > 0)
				{
					errorMessage = error;					
					return;
				}

				if (!CheckMetadataPrefix(prefix))
				{
					errorMessage = @"<error code=""badResumptionToken"">'" + resToken + "' is not a valid resumptionToken.</error>";
					return;
				}

				select = "id >= '" + startAtID + "'";
			}

			if (from != null && from.Length > 0)
			{
				if (select.Length != 0)
					select += "AND ";

				select += "lastModDate >= " + dateDelimiter + from + dateDelimiter; 
			}

			if (until != null && until.Length > 0)
			{
				if (select.Length != 0)
					select += "AND ";

				select += "lastModDate <= " + dateDelimiter + until + dateDelimiter;
			}
		

			// Select a subset of all records and sort by id
			DataView dv = new DataView(ds.Tables["record"]);
			dv.Sort = "id";
			dv.RowFilter = select;

			if (dv.Count == 0)
			{
				errorMessage = @"<error code=""noRecordsMatch"">No records match the given set, from, or until parameters.</error>";
			}
			else
			{
				System.Text.StringBuilder head = new System.Text.StringBuilder();
						
				int maxRecords = int.Parse(Application["MaxListIdentifiers"].ToString());
				int i;
				for (i = 0; i < dv.Count && i < maxRecords; i++)
				{
					head.Append("\t\t<header>\n");

					DataRowView row = dv[i];

					string date = Util.GetDate(row["lastModDate"].ToString());

					head.Append("\t\t\t<identifier>" + BuildOAIIdentifier(row["id"].ToString()) + "</identifier>\n" +
						"\t\t\t<datestamp>" + date + "</datestamp>\n"); 

					// Sets are not supported, so don't worry about <setSpec> tags

					head.Append("\t\t</header>\n");
				}

				headers = head.ToString();

				// Produce a resumption token if not all records were presented
				if (i < dv.Count)
				{
					string nextStartAtID = dv[i].Row["id"].ToString();
					nextResumptionToken = Util.BuildResumptionTokenWithOaiId(until, from, 
						theSet, prefix, nextStartAtID, Util.NowUTC(), "", "", "");
				}				
			}

		}


		/// <summary>
		/// Builds a valid OAI Identifier from the local ID.
		/// </summary>
		/// <param name="localId"></param>
		/// <returns>Complete OAI identifier</returns>
		private string BuildOAIIdentifier(string localId)
		{
			return "oai:" + Application["NamespaceIdentifier"] + ":" + localId;
		}

		/// <summary>
		/// Checks to see if given metadataPrefix is supported.
		/// </summary>
		/// <param name="prefix"></param>
		/// <returns>Returns true if metadataPrefix is supported, false otherwise.</returns>
		private bool CheckMetadataPrefix(string prefix)
		{
			ArrayList acceptFormats = (ArrayList)Application["AcceptFormats"];
			foreach (Global.AcceptFormats af in acceptFormats)
			{
				if (af.Format == prefix)
					return true;
			}
			return false;
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);

			// Make sure necessary params were sent
			CheckParameters();

			if (errorMessage == null)
			{
				RetrieveIdentifiers(metadataPrefix, fromDate, untilDate, setSpec, resumptionToken);
			}
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    

		}
		#endregion
	}
}
