/* Open Source License

Copyright 2005 by Frank McCown
All rights reserved.

Developed by:  Frank McCown
               Old Dominion University
               http://www.cs.odu.edu/~fmccown

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal with the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

    - Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimers.
    - Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimers in the
      documentation and/or other materials provided with the distribution.
    - Neither the names of Open Archives Initiative Metadata Harvesting
      Project, Old Dominion University, nor the names of
      its contributors may be used to endorse or promote products derived
      from this Software without specific prior written permission.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS WITH THE SOFTWARE.

*/

using System;

namespace oaidemo
{
	/// <summary>
	/// Summary description for util.
	/// </summary>
	public class Util
	{
		public Util()
		{
		}

		/// <summary>
		/// Ensures id is a valid URI (doesn't contain any excluded characters).
		/// </summary>
		/// <param name="id"></param>
		/// <returns>true if id is a valid URI, false otherwise.</returns>
		public static bool CheckId(string id)
		{
			try
			{
				Uri uri = new Uri(id);
				return true;
			}
			catch (UriFormatException)
			{
				return false;
			}
		}

		/// <summary>
		/// Parses an identifier into its scheme, namespaceIdentifier, and localIdentifier.
		/// Expected format for identifier => scheme ":" namespaceIdentifier ":" localIdentifier
		/// </summary>
		/// <param name="identifier"></param>
		/// <param name="scheme"></param>
		/// <param name="namespaceIdentifier"></param>
		/// <param name="localIdentifier"></param>
		/// <returns>true if parsing is successful, false otherwise.</returns>
		public static bool ParseOAIIdentifier(string identifier, out string scheme, out string namespaceIdentifier,
			out string localIdentifier)
		{
			char[] seperator = {':'};
			string[] parts = identifier.Split(seperator);

			if (parts.Length != 3)
			{
				scheme = "";   
				namespaceIdentifier = "";
				localIdentifier = "";
				return false;
			}

			scheme = parts[0];
			namespaceIdentifier = parts[1];
			localIdentifier = parts[2];

			return true;
		}


		/// <summary>
		/// Determines if date is in YYYY-MM-DD (ISO8601) format
		/// </summary>
		/// <param name="date"></param>
		/// <returns>True if date is in YYYY-MM-DD format, false otherwise.</returns>
		public static bool ValidDate(string date)
		{
			string pattern = @"^\d\d\d\d-\d\d-\d\d$";
			System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(pattern);
			System.Text.RegularExpressions.Match m = rx.Match(date);
			return m.Success;
		}

		/// <summary>
		/// According to http://www.openarchives.org/OAI/2.0/guidelines-static-repository.2002-11-11.htm
		/// The "datestamp" of records must be of the ISO8601 form YYYY-MM-DD.
		/// </summary>
		/// <param name="date"></param>
		/// <returns>Date in YYY-MM-DD format.</returns>
		public static string GetDate(string date)
		{
			DateTime d = DateTime.Parse(date);
			return d.ToString("yyyy-MM-dd");
		}


		/// <summary>
		/// Build a resumptionToken string from the given parts (with the unique ID).
		/// The resumptionToken syntax is:
		/// 	until|from|set|metadataPrefix|startingID|dateTimeStamp
		/// NOTE: cursor has to be a non-negative integer
		/// </summary>
		/// <param name="untilDate"></param>
		/// <param name="fromDate"></param>
		/// <param name="setSpec"></param>
		/// <param name="metadataPrefix"></param>
		/// <param name="startAtID"></param>
		/// <param name="timeStamp"></param>
		/// <param name="expDate"></param>
		/// <param name="size"></param>
		/// <param name="cursor"></param>
		/// <returns>A resumptionToken (including the tags and attributes)</returns>
		public static string BuildResumptionTokenWithOaiId(string untilDate, string fromDate, string setSpec, 
			string metadataPrefix, string startAtID, string timeStamp, string expDate, string size, string cursor)
		{
			string token = "\t<resumptionToken";
			
			if (expDate.Length > 0)
				token += " expirationDate='" + expDate + "' ";
			if (size.Length > 0)
				token += " completeListSize='" + size + "' ";
			if (cursor.Length > 0)
				token += " cursor='" + cursor + "' ";

			// If there is no "startAtID", then an empty resumptionToken needed    
			if (startAtID.Length == 0) 
				token += "/>\n";
			else
				token += ">" + untilDate + "|" + fromDate + "|" + setSpec + "|" + metadataPrefix + "|" +
					startAtID + "|" + timeStamp + "</resumptionToken>\n";

			return token;
		}

		/// <summary>
		/// Parses a resumption token into its various parts:
		/// until|from|set|metadataPrefix|startingPosition|dateTimeStamp
		/// </summary>
		/// <param name="expTimeStamp"></param>
		/// <param name="resumptionToken"></param>
		/// <param name="untilDate"></param>
		/// <param name="fromDate"></param>
		/// <param name="setSpec"></param>
		/// <param name="metadataPrefix"></param>
		/// <param name="startAtID"></param>
		/// <returns>If the resumptionToken cannot be parsed or has expired, an error string is returned,
		/// otherwise an empty string is returned.</returns>
		public static string ParseResumptionTokenWithOaiId(string expTimeStamp, 
			string resumptionToken, out string untilDate, out string fromDate, 
			out string setSpec, out string metadataPrefix, out string startAtID)
		{
			string token = "";

			untilDate = null;
			fromDate = null;
			setSpec = null;
			metadataPrefix = null;
			startAtID = null;

			// Parse Resumption Token (contains all parameters plus counter for where left off last transaction)
			if (resumptionToken.Length > 0) 
			{
				char[] seperator = {'|'};
				string[] parts = resumptionToken.Split(seperator);
				if (parts.Length != 6)
				{
					token = @"<error code=""badResumptionToken"">'" + resumptionToken + "' is not a valid resumptionToken.</error>";
				}
				else
				{
					untilDate = parts[0];
					fromDate = parts[1];
					setSpec = parts[2];
					metadataPrefix = parts[3];
					startAtID = parts[4];

					// Get rid of T and Z
					string rTIssue = parts[5].Replace("T", " ");
					rTIssue = rTIssue.Substring(0, rTIssue.Length - 1);

					try
					{
						// See if token has expired
						DateTime issue = DateTime.Parse(rTIssue);
						DateTime exp = DateTime.Parse(expTimeStamp);

						if (DateTime.Compare(issue, exp) > 0)
						{
							token = @"<error code=""badResumptionToken"">'" + resumptionToken + "' has expired.</error>";
						}
						else if ((untilDate.Length > 0 && !ValidDate(untilDate)) || 
							(fromDate.Length > 0 && !ValidDate(fromDate)))
						{
							token = @"<error code=""badResumptionToken"">'" + resumptionToken + "' is not a valid resumptionToken.</error>";
						}
					}
					catch (Exception)
					{
						token = @"<error code=""badResumptionToken"">'" + resumptionToken + "' is not a valid resumptionToken.</error>";                        
					}					
				}
			}

			return token;
		}


		/// <summary>
		/// Returns a valid OAI timestamp based from current Universal Time.  Example: 2004-12-04T19:55:06Z
		/// </summary>
		/// <returns></returns>
		public static string NowUTC()
		{
			// Creates and initializes a DateTimeFormatInfo associated with the en-US culture.
			System.Globalization.DateTimeFormatInfo myDTFI = new System.Globalization.CultureInfo( "en-US", false ).DateTimeFormat;

			// Generate valid OAI timestamp.
			// SortableDateTimePattern (based on ISO 8601) using local time: yyyy'-'MM'-'ddTHH':'mm':'ss'
			return DateTime.UtcNow.ToString("s", myDTFI) + "Z";
		}
	}
}
